@@ -92,16 +92,37 @@ UMK = "Metric kmph"
 UI = "Imperial mph"
 UIK = "Imperial knots"
 
+HIDE_NULL_GNSS = True  # whether to hide GNSS with zero snr in graph view
+GLONASS_NMEA = True  # use GLONASS NMEA SVID (65-96) rather than slot (1-24)
+
+# GNSS graph color scheme
+# ("GPS","SBAS","Galileo","BeiDou""IMES","QZSS","GLONASS")
+GNSS_COLS = (
+    "royalblue",
+    "lightslategray",
+    "green",
+    "purple",
+    "gray",
+    "gray",
+    "indianred",
+)
+
 # List of tags to highlight in console
 TAGS = [
     ("ACK-ACK", "green2"),
     ("ACK-NAK", "orange red"),
     ("CFG-MSG", "cyan"),
     ("DTM", "deepskyblue"),
+    ("GAQ", "pink"),
+    ("GBQ", "pink"),
     ("GBS", "pink"),
     ("GGA", "orange"),
     ("GLL", "orange"),
+    ("GLQ", "pink"),
+    ("GNQ", "pink"),
     ("GNS", "orange"),
+    ("GQQ", "pink"),
+    ("GRS", "yellow"),
     ("GSA", "green2"),
     ("GST", "mediumpurple2"),
     ("GSV", "yellow"),
@@ -110,20 +131,21 @@ TAGS = [
     ("INF-WARNING", "orange"),
     ("LOG", "skyblue1"),
     ("MON", "skyblue1"),
+    ("NAV-ATT", "yellow"),
     ("NAV-AOPSTATUS", "yellow"),
-    ("NAV-COV", "yellow"),
     ("NAV-CLOCK", "cyan"),
+    ("NAV-COV", "yellow"),
     ("NAV-DGPS", "yellow"),
     ("NAV-DOP", "mediumpurple2"),
     ("NAV-EELL", "yellow"),
     ("NAV-EOE", "yellow"),
     ("NAV-GEOFENCE", "yellow"),
+    ("NAV-HPPOSECEF", "orange"),
+    ("NAV-HPPOSLLH", "orange"),
     ("NAV-ODO", "deepskyblue"),
     ("NAV-ORB", "yellow"),
     ("NAV-POSECEF", "orange"),
     ("NAV-POSLLH", "orange"),
-    ("NAV-HPPOSECEF", "orange"),
-    ("NAV-HPPOSLLH", "orange"),
     ("NAV-PVT", "orange"),
     ("NAV-SAT", "yellow"),
     ("NAV-SBAS", "yellow"),
@@ -141,6 +163,7 @@ TAGS = [
     ("NAV-TIMEUTC", "cyan"),
     ("NAV-VELECEF", "deepskyblue"),
     ("NAV-VELNED", "deepskyblue"),
+    ("RLM", "pink"),
     ("RMC", "orange"),
     ("RXM", "skyblue1"),
     ("TXT", "lightgrey"),
@@ -152,14 +175,18 @@ TAGS = [
     ("UBX06", "orange"),
     ("VLW", "deepskyblue"),
     ("VTG", "deepskyblue"),
-    ("xb5b", "lightblue1"),
     ("ZDA", "cyan"),
+    ("xb5b", "lightblue1"),
 ]
 
 
 def deg2rad(deg: float) -> float:
     """
     Convert degrees to radians
+
+    :param float deg: degrees
+    :return radians
+    :rtype float
     """
 
     if not isinstance(deg, (float, int)):
@@ -170,6 +197,11 @@ def deg2rad(deg: float) -> float:
 def cel2cart(elevation: float, azimuth: float) -> (float, float):
     """
     Convert celestial coordinates (degrees) to Cartesian coordinates
+
+    :param float elevation
+    :param float azimuth
+    :return cartesian x,y coordinates
+    :rtype tuple
     """
 
     if not (isinstance(elevation, (float, int)) and isinstance(azimuth, (float, int))):
@@ -184,6 +216,10 @@ def cel2cart(elevation: float, azimuth: float) -> (float, float):
 def deg2dms(degrees: float, latlon: str) -> str:
     """
     Convert decimal degrees to degrees minutes seconds string
+
+    :param float degrees
+    :return degrees as d.m.s formatted string
+    :rtype str
     """
 
     if not isinstance(degrees, (float, int)):
@@ -210,6 +246,11 @@ def deg2dms(degrees: float, latlon: str) -> str:
 def deg2dmm(degrees: float, latlon: str) -> str:
     """
     Convert decimal degrees to degrees decimal minutes string
+
+    :param float: degrees
+    :param str: latlon: whether degrees refer to lat or lon
+    :return degrees as dm.m formatted string
+    :rtype str
     """
 
     if not isinstance(degrees, (float, int)):
@@ -227,6 +268,10 @@ def deg2dmm(degrees: float, latlon: str) -> str:
 def m2ft(meters: float) -> float:
     """
     Convert meters to feet
+
+    :param float meters
+    :return feet
+    :rtype float
     """
 
     if not isinstance(meters, (float, int)):
@@ -237,6 +282,11 @@ def m2ft(meters: float) -> float:
 def ft2m(feet: float) -> float:
     """
     Convert feet to meters
+
+
+    :param float feet
+    :return elevation in meters
+    :rtype float
     """
 
     if not isinstance(feet, (float, int)):
@@ -247,6 +297,10 @@ def ft2m(feet: float) -> float:
 def ms2kmph(ms: float) -> float:
     """
     Convert meters per second to kilometers per hour
+
+    :param float ms
+    :return speed in kmph
+    :rtype float
     """
 
     if not isinstance(ms, (float, int)):
@@ -257,6 +311,10 @@ def ms2kmph(ms: float) -> float:
 def ms2mph(ms: float) -> float:
     """
     Convert meters per second to miles per hour
+
+    :param float ms
+    :return speed in mph
+    :rtype float
     """
 
     if not isinstance(ms, (float, int)):
@@ -267,6 +325,10 @@ def ms2mph(ms: float) -> float:
 def ms2knots(ms: float) -> float:
     """
     Convert meters per second to knots
+
+    :param float: ms
+    :return speed in knots
+    :rtype float
     """
 
     if not isinstance(ms, (float, int)):
@@ -277,6 +339,10 @@ def ms2knots(ms: float) -> float:
 def kmph2ms(kmph: float) -> float:
     """
     Convert kilometers per hour to meters per second
+
+    :param float: kmph
+    :return speed in m/s
+    :rtype float
     """
 
     if not isinstance(kmph, (float, int)):
@@ -287,6 +353,10 @@ def kmph2ms(kmph: float) -> float:
 def knots2ms(knots: float) -> float:
     """
     Convert knots to meters per second
+
+    :param float: knots
+    :return speed in m/s
+    :rtype float
     """
 
     if not isinstance(knots, (float, int)):
@@ -297,6 +367,13 @@ def knots2ms(knots: float) -> float:
 def pos2iso6709(lat: float, lon: float, alt: float, crs: str = "WGS_84") -> str:
     """
     convert decimal degrees and alt to iso6709 format
+
+    :param float lat
+    :param float lon
+    :param float alt: altitude
+    :param float crs: coordinate reference system
+    :return position in iso6709 format
+    :rtype str
     """
 
     if not (
@@ -325,6 +402,12 @@ def pos2iso6709(lat: float, lon: float, alt: float, crs: str = "WGS_84") -> str:
 def hsv2rgb(h: float, s: float, v: float) -> str:
     """
     Convert HSV values (in range 0-1) to RGB color string.
+
+    :param float h: hue
+    :param float s: saturation
+    :param float v: value
+    :return rgb color value
+    :rtype str
     """
 
     if s == 0.0:
@@ -357,6 +440,10 @@ def snr2col(snr: int) -> str:
     """
     Convert satellite signal-to-noise ratio to a color
     high = green, low = red
+
+    :param int snr: signal to noise ratio as integer
+    :return rgb color string
+    :rtype str
     """
 
     return hsv2rgb(snr / (MAX_SNR * 2.5), 0.8, 0.8)
@@ -365,6 +452,10 @@ def snr2col(snr: int) -> str:
 def nmea2latlon(data: types.talker) -> (float, float):
     """
     Convert parsed NMEA sentence to decimal lat, lon
+
+    :param pynmea2.types.talker data: parsed NMEA sentence
+    :return (lat, lon)
+    :rtype tuple
     """
 
     if data.lat == "":
@@ -380,3 +471,29 @@ def nmea2latlon(data: types.talker) -> (float, float):
         lonmin = float(data.lon[3:])
         lon = (londeg + lonmin / 60) * (-1 if data.lon_dir == "W" else 1)
     return (lat, lon)
+
+
+def svid2gnssid(svid) -> int:
+    """
+    Derive gnssId from svid numbering range
+
+    :param int svid
+    :return gnssId
+    :rtype int
+    """
+
+    if 120 <= svid <= 158:
+        gnssId = 1  # SBAS
+    elif 211 <= svid <= 246:
+        gnssId = 2  # Galileo
+    elif (159 <= svid <= 163) or (33 <= svid <= 64):
+        gnssId = 3  # Beidou
+    elif 173 <= svid <= 182:
+        gnssId = 4  # IMES
+    elif 193 <= svid <= 202:
+        gnssId = 5  # QZSS
+    elif (65 <= svid <= 96) or svid == 255:
+        gnssId = 6  # GLONASS
+    else:
+        gnssId = 0  # GPS
+    return gnssId
