@@ -10,10 +10,10 @@ Created on 30 Sep 2020
 # pylint: disable=invalid-name
 
 from datetime import datetime
-from pyubx2 import UBXMessage, POLL, UBX_MSGIDS, UBX_CONFIG_MESSAGES
+from pyubx2 import UBXMessage, POLL, UBX_MSGIDS
+from pyubx2.ubxhelpers import itow2utc, gpsfix2str
+from .globals import svid2gnssid, GLONASS_NMEA, HIDE_NULL_GNSS
 
-CFG_MSG_OFF = b"\x00\x00\x00\x00\x00\x01"
-CFG_MSG_ON = b"\x01\x01\x01\x01\x01\x01"
 BOTH = 3
 UBX = 1
 NMEA = 2
@@ -28,7 +28,7 @@ class UBXHandler:
         """
         Constructor.
 
-        :param app: reference to main tkinter application
+        :param object app: reference to main tkinter application
         """
 
         self.__app = app  # Reference to main application class
@@ -68,7 +68,7 @@ class UBXHandler:
         may take several seconds to arrive, particularly in
         heavy traffic.
 
-        :param serial
+        :param serial: serial
         """
 
         for portid in (0, 1, 3, 4):  # I2C, UART1, USB, SPI
@@ -83,9 +83,9 @@ class UBXHandler:
         """
         Process UBX message type
 
-        :param data: bytes
-
-        :return UBX message as UBXMessage:
+        :param bytes data: raw data
+        :return UBXMessage:
+        :rtype: UBXMessage
         """
 
         parsed_data = UBXMessage.parse(data, False)
@@ -127,8 +127,8 @@ class UBXHandler:
         """
         Write the incoming data to the console in raw or parsed format.
 
-        :param raw_data: bytes
-        :param parsed_data: UBXMessage
+        :param bytes raw_data: raw data
+        :param UBXMessage parsed_data: UBXMessage
         """
 
         if self.__app.frm_settings.get_settings()["raw"]:
@@ -140,36 +140,41 @@ class UBXHandler:
         """
         Process CFG-MSG sentence - UBX message configuration.
 
-        :param data: UBXMessage
+        :param UBXMessage data: ACK_ACK parsed message
         """
 
-        msgtype = UBX_MSGIDS[self.msgclass2bytes(data.clsID, data.msgID)]
+        (ubxClass, ubxID) = UBXMessage.msgclass2bytes(data.clsID, data.msgID)
 
         # update the UBX config panel
         if self.__app.dlg_ubxconfig is not None:
-            self.__app.dlg_ubxconfig.update("ACK-ACK", msgtype=msgtype)
+            self.__app.dlg_ubxconfig.update(
+                "ACK-ACK", msgtype=UBX_MSGIDS[ubxClass + ubxID]
+            )
 
     def _process_ACK_NAK(self, data: UBXMessage):
         """
         Process CFG-MSG sentence - UBX message configuration.
 
-        :param data: UBXMessage
+        :param UBXMessage data: ACK_NAK parsed message
         """
 
-        msgtype = UBX_MSGIDS[self.msgclass2bytes(data.clsID, data.msgID)]
+        (ubxClass, ubxID) = UBXMessage.msgclass2bytes(data.clsID, data.msgID)
 
         # update the UBX config panel
         if self.__app.dlg_ubxconfig is not None:
-            self.__app.dlg_ubxconfig.update("ACK-NAK", msgtype=msgtype)
+            self.__app.dlg_ubxconfig.update(
+                "ACK-NAK", msgtype=UBX_MSGIDS[ubxClass + ubxID]
+            )
 
     def _process_CFG_MSG(self, data: UBXMessage):
         """
         Process CFG-MSG sentence - UBX message configuration.
 
-        :param data: UBXMessage
+        :param UBXMessage data: CFG-MSG parsed message
         """
 
-        msgtype = UBX_CONFIG_MESSAGES[self.msgclass2bytes(data.msgClass, data.msgID)]
+        (ubxClass, ubxID) = UBXMessage.msgclass2bytes(data.msgClass, data.msgID)
+
         ddcrate = data.rateDDC
         uart1rate = data.rateUART1
         uart2rate = data.rateUART2
@@ -180,7 +185,7 @@ class UBXHandler:
         if self.__app.dlg_ubxconfig is not None:
             self.__app.dlg_ubxconfig.update(
                 "CFG-MSG",
-                msgtype=msgtype,
+                msgtype=UBX_MSGIDS[ubxClass + ubxID],
                 ddcrate=ddcrate,
                 uart1rate=uart1rate,
                 uart2rate=uart2rate,
@@ -188,11 +193,11 @@ class UBXHandler:
                 spirate=spirate,
             )
 
-    def _process_CFG_INF(self, data: UBXMessage):
+    def _process_CFG_INF(self, data: UBXMessage):  # pylint: disable=unused-argument
         """
         Process CFG-INF sentence - UBX info message configuration.
 
-        :param data: UBXMessage
+        :param UBXMessage data: CFG-INF parsed message
         """
 
         # update the UBX config panel
@@ -203,7 +208,7 @@ class UBXHandler:
         """
         Process CFG-PRT sentence - UBX port configuration.
 
-        :param data: UBXMessage
+        :param UBXMessage data: CFG-PRT parsed message
         """
 
         self.ubx_portid = data.portID
@@ -225,11 +230,11 @@ class UBXHandler:
         """
         Process NAV-LLH sentence - Latitude, Longitude, Height.
 
-        :param data: UBXMessage
+        :param UBXMessage data: NAV-POSLLH parsed message
         """
 
         try:
-            self.utc = UBXMessage.itow2utc(data.iTOW)
+            self.utc = itow2utc(data.iTOW)
             self.lat = data.lat / 10 ** 7
             self.lon = data.lon / 10 ** 7
             self.alt = data.hMSL / 1000
@@ -254,11 +259,11 @@ class UBXHandler:
         """
         Process NAV-PVT sentence -  Navigation position velocity time solution.
 
-        :param data: UBXMessage
+        :param UBXMessage data: NAV-PVT parsed message
         """
 
         try:
-            self.utc = UBXMessage.itow2utc(data.iTOW)
+            self.utc = itow2utc(data.iTOW)
             self.lat = data.lat / 10 ** 7
             self.lon = data.lon / 10 ** 7
             self.alt = data.hMSL / 1000
@@ -268,7 +273,7 @@ class UBXHandler:
             self.sip = data.numSV
             self.speed = data.gSpeed / 1000  # m/s
             self.track = data.headMot / 10 ** 5
-            fix = UBXMessage.gpsfix2str(data.fixType)
+            fix = gpsfix2str(data.fixType)
             self.__app.frm_banner.update_banner(
                 time=self.utc,
                 lat=self.lat,
@@ -324,7 +329,7 @@ class UBXHandler:
         """
         Process NAV-VELNED sentence - Velocity Solution in North East Down format.
 
-        :param data: UBXMessage
+        :param UBXMessage data: NAV-VELNED parsed message
         """
 
         try:
@@ -339,23 +344,34 @@ class UBXHandler:
         """
         Process NAV-SAT sentences - Space Vehicle Information.
 
-        :param data: UBXMessage
+        NB: For consistency with NMEA GSV and UBX NAV-SVINFO message types,
+        this uses the NMEA SVID numbering range for GLONASS satellites
+        (65 - 96) rather than the Slot ID (1-24) by default.
+        To change this, set the GLONASS_NMEA flag in globals.py to False.
+
+        :param UBXMessage data: NAV-SAT parsed message
         """
 
         try:
             self.gsv_data = []
             num_siv = int(data.numCh)
-            self.__app.frm_banner.update_banner(siv=num_siv)
 
             for i in range(num_siv):
                 idx = "_{0:0=2d}".format(i + 1)
+                gnssId = getattr(data, "gnssId" + idx)
                 svid = getattr(data, "svId" + idx)
+                # use NMEA GLONASS numbering (65-96) rather than slotID (1-24)
+                if gnssId == 6 and svid < 25 and svid != 255 and GLONASS_NMEA:
+                    svid += 64
                 elev = getattr(data, "elev" + idx)
                 azim = getattr(data, "azim" + idx)
                 cno = getattr(data, "cno" + idx)
-                self.gsv_data.append((svid, elev, azim, cno))
+                if cno == 0 and HIDE_NULL_GNSS:  # omit sats with zero cno
+                    continue
+                self.gsv_data.append((gnssId, svid, elev, azim, cno))
+            self.__app.frm_banner.update_banner(siv=len(self.gsv_data))
             self.__app.frm_satview.update_sats(self.gsv_data)
-            self.__app.frm_graphview.update_graph(self.gsv_data, num_siv)
+            self.__app.frm_graphview.update_graph(self.gsv_data, len(self.gsv_data))
         except ValueError:
             # self.__app.set_status(ube.UBXMessageError(err), "red")
             pass
@@ -364,23 +380,28 @@ class UBXHandler:
         """
         Process NAV-SVINFO sentences - Space Vehicle Information.
 
-        :param data: UBXMessage
+        NB: Since UBX Gen8 this message is deprecated in favour of NAV-SAT
+
+        :param UBXMessage data: NAV-SVINFO parsed message
         """
 
         try:
             self.gsv_data = []
             num_siv = int(data.numCh)
-            self.__app.frm_banner.update_banner(siv=num_siv)
 
             for i in range(num_siv):
                 idx = "_{0:0=2d}".format(i + 1)
                 svid = getattr(data, "svid" + idx)
+                gnssId = svid2gnssid(svid)  # derive gnssId from svid
                 elev = getattr(data, "elev" + idx)
                 azim = getattr(data, "azim" + idx)
                 cno = getattr(data, "cno" + idx)
-                self.gsv_data.append((svid, elev, azim, cno))
+                if cno == 0 and HIDE_NULL_GNSS:  # omit sats with zero cno
+                    continue
+                self.gsv_data.append((gnssId, svid, elev, azim, cno))
+            self.__app.frm_banner.update_banner(siv=siv)
             self.__app.frm_satview.update_sats(self.gsv_data)
-            self.__app.frm_graphview.update_graph(self.gsv_data, num_siv)
+            self.__app.frm_graphview.update_graph(self.gsv_data, len(self.gsv_data))
         except ValueError:
             # self.__app.set_status(ube.UBXMessageError(err), "red")
             pass
@@ -389,13 +410,13 @@ class UBXHandler:
         """
         Process NAV-SOL sentence - Navigation Solution.
 
-        :param data: UBXMessage
+        :param UBXMessage data: NAV-SOL parsed message
         """
 
         try:
             self.pdop = data.pDOP / 100
             self.sip = data.numSV
-            fix = UBXMessage.gpsfix2str(data.gpsFix)
+            fix = gpsfix2str(data.gpsFix)
 
             self.__app.frm_banner.update_banner(dop=self.pdop, fix=fix, sip=self.sip)
         except ValueError:
@@ -406,7 +427,7 @@ class UBXHandler:
         """
         Process NAV-DOP sentence - Dilution of Precision.
 
-        :param data: UBXMessage
+        :param UBXMessage data: NAV-DOP parsed message
         """
 
         try:
@@ -425,7 +446,7 @@ class UBXHandler:
         """
         Process MON-VER sentence - Receiver Software / Hardware version information.
 
-        :param data: UBXMessage
+        :param UBXMessage data: MON-VER parsed message
         """
 
         exts = []
@@ -479,7 +500,7 @@ class UBXHandler:
         """
         Process MON-HW sentence - Receiver Hardware status.
 
-        :param data: UBXMessage
+        :param UBXMessage data: MON-HW parsed message
         """
 
         ant_status = getattr(data, "aStatus", 1)
@@ -490,18 +511,3 @@ class UBXHandler:
             self.__app.dlg_ubxconfig.update(
                 "MON-HW", antstatus=ant_status, antpower=ant_power
             )
-
-    @staticmethod
-    def msgclass2bytes(msgClass: int, msgID: int) -> bytes:
-        """
-        Convert message class/id integers to bytes.
-
-        :param msgCLass: int
-        :param msgID: int
-
-        :return message class as string: str
-        """
-
-        msgClass = msgClass.to_bytes(1, byteorder="little", signed=False)
-        msgID = msgID.to_bytes(1, byteorder="little", signed=False)
-        return msgClass + msgID
